import "source-map-support/register";
import Keyv from "keyv";
import type { Collection, CountDocumentsOptions, Filter, FindCursor, FindOptions, WithId, Document, DistinctOptions } from "mongodb";
export default class MongoDBCaching<TSchema extends Document = Document, TContext = any> {
    /**
     * MongoDB collection
     */
    collection: Collection<TSchema>;
    keyv: Keyv;
    context?: TContext;
    constructor(
    /**
     * MongoDB collection
     */
    collection: Collection<TSchema>, keyvOptions?: Keyv.Options<any> & Record<string, unknown>);
    /**
     * Sets the context for the Cache
     * @param context Context to set
     */
    setContext(context: TContext): void;
    /**
     * Fetches the first document that matches the filter
     * @param filter Query for find Operation
     * @param options Options for find Operation
     */
    findOne(filter?: Filter<TSchema>, options?: {
        findOptions: FindOptions<Document>;
        ttl?: number;
    }): Promise<Awaited<TSchema | null>>;
    /**
     *
     * @param filter Query for find Operation
     * @param options Options for find Operation
     */
    find(filter?: Filter<TSchema>, options?: {
        findOptions?: FindOptions<Document>;
        cursor?: (c: FindCursor<WithId<TSchema>>) => Promise<WithId<TSchema>[]>;
        ttl?: number;
    }): Promise<WithId<TSchema>[]>;
    countDocuments(filter?: Filter<TSchema>, options?: {
        countOptions?: CountDocumentsOptions;
        ttl?: number;
    }): Promise<number>;
    distinct<Key extends keyof WithId<TSchema>>(key: Key, filter: Filter<TSchema>, options?: {
        distinctOptions?: DistinctOptions;
        ttl?: number;
    }): Promise<any>;
    clear(): Promise<void>;
    /**
     * Throttles a function to only be called once per given key until the function is done
     */
    activeThrottles: Map<string, Promise<any>>;
    private throttleFunction;
    getCacheKey(...args: any[]): string;
}
