"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("source-map-support/register");
const keyv_1 = __importDefault(require("keyv"));
class MongoDBCaching {
    constructor(
    /**
     * MongoDB collection
     */
    collection, keyvOptions) {
        this.collection = collection;
        /**
         * Throttles a function to only be called once per given key until the function is done
         */
        this.activeThrottles = new Map();
        this.keyv = new keyv_1.default(keyvOptions);
    }
    /**
     * Sets the context for the Cache
     * @param context Context to set
     */
    setContext(context) {
        this.context = context;
    }
    /**
     * Fetches the first document that matches the filter
     * @param filter Query for find Operation
     * @param options Options for find Operation
     */
    findOne(filter = {}, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheKey = "findOne-" + this.getCacheKey(filter, options);
            return this.throttleFunction(cacheKey, () => __awaiter(this, void 0, void 0, function* () {
                const cacheDoc = yield this.keyv.get(cacheKey);
                if (cacheDoc)
                    return cacheDoc;
                const res = yield this.collection.findOne(filter, options === null || options === void 0 ? void 0 : options.findOptions);
                yield this.keyv.set(cacheKey, res, options === null || options === void 0 ? void 0 : options.ttl);
                return res;
            }));
        });
    }
    /**
     *
     * @param filter Query for find Operation
     * @param options Options for find Operation
     */
    find(filter = {}, options) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const cacheKeyOptions = options
                ? Object.assign(Object.assign({}, options), { cursor: (_a = options.cursor) === null || _a === void 0 ? void 0 : _a.toString() }) : undefined;
            const cacheKey = "find-" + this.getCacheKey(filter, cacheKeyOptions);
            return this.throttleFunction(cacheKey, () => __awaiter(this, void 0, void 0, function* () {
                var _b;
                const cacheDoc = yield this.keyv.get(cacheKey);
                if (cacheDoc)
                    return cacheDoc;
                const res = (yield ((_b = options === null || options === void 0 ? void 0 : options.cursor) === null || _b === void 0 ? void 0 : _b.call(options, this.collection.find(filter, options.findOptions)))) ||
                    (yield this.collection.find(filter, options === null || options === void 0 ? void 0 : options.findOptions).toArray());
                yield this.keyv.set(cacheKey, res, options === null || options === void 0 ? void 0 : options.ttl);
                return res;
            }));
        });
    }
    countDocuments(filter = {}, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheKey = "countDocuments-" + this.getCacheKey(filter);
            return this.throttleFunction(cacheKey, () => __awaiter(this, void 0, void 0, function* () {
                const cacheDoc = yield this.keyv.get(cacheKey);
                if (cacheDoc)
                    return cacheDoc;
                const res = yield this.collection.countDocuments(filter, (options === null || options === void 0 ? void 0 : options.countOptions) || {});
                yield this.keyv.set(cacheKey, res, options === null || options === void 0 ? void 0 : options.ttl);
                return res;
            }));
        });
    }
    distinct(key, filter, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const cacheKey = `distinct-${key.toString()}-` + this.getCacheKey(filter);
            return this.throttleFunction(cacheKey, () => __awaiter(this, void 0, void 0, function* () {
                const cacheDoc = yield this.keyv.get(cacheKey);
                if (cacheDoc)
                    return cacheDoc;
                const res = yield this.collection.distinct(key, filter, options === null || options === void 0 ? void 0 : options.distinctOptions);
                yield this.keyv.set(cacheKey, res, options === null || options === void 0 ? void 0 : options.ttl);
                return res;
            }));
        });
    }
    clear() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.keyv.clear();
        });
    }
    throttleFunction(name, callback) {
        if (this.activeThrottles.has(name))
            return this.activeThrottles.get(name);
        const promise = callback();
        this.activeThrottles.set(name, promise);
        promise.finally(() => {
            this.activeThrottles.delete(name);
        });
        return promise;
    }
    getCacheKey(...args) {
        const elements = args.filter(arg => typeof arg === "object");
        return JSON.stringify(elements.length === 1 ? elements[0] : elements);
    }
}
exports.default = MongoDBCaching;
//# sourceMappingURL=index.js.map